#ifndef __Mt_Avs25Wrapper_H__
#define __Mt_Avs25Wrapper_H__

#include "avisynth.h"
#include "params.h"

#include "../common/frame/frame.h"
#include "../common/clip/clip.h"
#include "../common/params/params.h"

namespace MaskTools { namespace Avs25 {

/* todo : add checks for colorspace, resolution & co */
template<class T>
class Wrapper_T : public T, public GenericVideoFilter {
public:
   PClip child2, child3;
   Wrapper_T(PClip child, PClip child2, PClip child3, const MaskTools::Parameters::Parameters &parameters, IScriptEnvironment* env) : T(parameters), GenericVideoFilter(child), child2(child2), child3(child3)
   {
      if ( error.length() )
         env->ThrowError(error.c_str());

      if ( T::inputClips.clip(0) == Clip::IN_PLACE )
      {
         if ( Parameters::Operator(Y) == Parameters::COPY ) Y = Parameters::NONE;
         if ( Parameters::Operator(U) == Parameters::COPY ) U = Parameters::NONE;
         if ( Parameters::Operator(V) == Parameters::COPY ) V = Parameters::NONE;
         if ( Parameters::Operator(Y) == Parameters::COPY_SECOND ) Y = Parameters::COPY;
         if ( Parameters::Operator(U) == Parameters::COPY_SECOND ) U = Parameters::COPY;
         if ( Parameters::Operator(V) == Parameters::COPY_SECOND ) V = Parameters::COPY;
      }
   }

   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env)
   {
      PClip childs[3] = { child, child2, child3 };
      PVideoFrame clips[4];
      Picture::Frame dst;

      /* no need to check if n + offset is a real frame, avs 2.5 does that for us */
      for ( int i = 0; i < 4; i++ )
      {
         switch ( Clip::Process(T::inputClips.clip(i)) )
         {
         case Clip::IN_PLACE:
            clips[i] = childs[T::inputClips.clip(i).idx()]->GetFrame(T::inputClips.clip(i).offset() + n, env);
            env->MakeWritable(&clips[i]);
            /* hack to request pointers in the right order ( *sigh* ) */
            dst = PVFToFrame(clips[0]);
            break;
         case Clip::CREATE:
            clips[i] = env->NewVideoFrame(vi);
            /* hack to request pointers in the right order ( *sigh* ) */
            dst = PVFToFrame(clips[0]);
            break;
         case Clip::SOURCE: clips[i] = childs[T::inputClips.clip(i).idx()]->GetFrame(T::inputClips.clip(i).offset() + n, env); break;
         }
      }

      ProcessFrame(dst, PVFToConstFrame(clips[1]), PVFToConstFrame(clips[2]), PVFToConstFrame(clips[3]));

      return clips[0];
   }
};

template<class T> AVSValue __cdecl Create_Filter_T(AVSValue args, void *user_data, IScriptEnvironment *env)
{
   return new Wrapper_T<T>(args[0].AsClip(), T::inputClips.IsClip(1) ? args[1].AsClip() : PClip(), T::inputClips.IsClip(2) ? args[2].AsClip() : PClip(), GetParameters(args, T::getSignature(), env), env);
}

template<class T> void AddFilter_T(IScriptEnvironment *env)
{
   env->AddFunction(T::getName().c_str(), SignatureToParamsSignature(T::getSignature()).c_str(), Create_Filter_T<T>, NULL);
}

} }

#endif
