#ifndef __Mt_Base_H__
#define __Mt_Base_H__

#include "../../common/common.h"
#include "../../common/functions/functions.h"
#include "../../common/params/params.h"
#include "../../common/constraints/constraints.h"
#include "../../common/frame/frame.h"

namespace MaskTools { 

class Filter {

protected:

   Constraints::CpuFlags flags;

   Parameters::Operator Y, U, V;

   int nXOffset, nYOffset, nXOffsetUV, nYOffsetUV;
   int nWidth, nHeight, nWidthUV, nHeightUV;
   int nRealWidth, nRealHeight;

   String error;

   Constraints::ProcessorList<PlaneProcessor>     processors;
   Constraints::ProcessorList<Functions::Prefetch> prefetches;

private:

   Constraints::ProcessorList<Functions::Memset>  memsets;
   Constraints::ProcessorList<Functions::Copy>    copies;

protected:

   void addProcessor(const Constraints::Processor<PlaneProcessor> &ppc);

   virtual void ProcessY(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1, Picture::CPlane &src2, Picture::CPlane &src3) = 0;
   virtual void ProcessU(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1, Picture::CPlane &src2, Picture::CPlane &src3) = 0;
   virtual void ProcessV(PlaneProcessor *processor, Picture::Plane &dst, Picture::CPlane &src1, Picture::CPlane &src2, Picture::CPlane &src3) = 0;

public:

   Filter(const Parameters::Parameters &parameters);

   void ProcessFrame(Picture::Frame &dst, Picture::CFrame &src1, Picture::CFrame &src2, Picture::CFrame &src3);

   static Parameters::Signature getSignature();
   static Clip::Inputs inputClips;

};

}

#endif
