#include "filter.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;

namespace MaskTools { namespace Filters { namespace Binarize {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nThreshold, nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nThreshold, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1, CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nThreshold, nWidthUV, nHeightUV);
}

Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   nThreshold = clip(parameters["threshold"].toInt(), 0, 255);

   /* add the processors */
   if ( parameters["upper"].toBool() )
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&upper_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_upper8_mmx), Constraint(CPU_MMX , 8, 1, 1, 1), 1));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_upper8_isse), Constraint(CPU_ISSE , 8, 1, 1, 1), 2));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_upper8_3dnow), Constraint(CPU_3DNOW , 8, 1, 1, 1), 3));
   }
   else
   {
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&lower_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_lower8_mmx), Constraint(CPU_MMX , 8, 1, 1, 1), 1));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_lower8_isse), Constraint(CPU_ISSE , 8, 1, 1, 1), 2));
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&Binarize_lower8_3dnow), Constraint(CPU_3DNOW , 8, 1, 1, 1), 3));
   }
}

String Filter::getName() { return "mt_binarize"; }
Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(128, "threshold"));
   signature.push_back(Parameter(false, "upper"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::IPOneFrame();

} } }
