#include "filter.h"
#include "../../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Blur { namespace Mapped {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), matrix, nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), matrix, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), src2.plane(), src2.pitch(), matrix, nWidthUV, nHeightUV);
}


Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   std::list<Symbol> coeffs = Parser::getDefaultParser().parse(parameters["kernel"].toString(), " ").getExpression();
   memset(matrix, 0, sizeof(matrix));
   for ( int i = 0; i < 9; i++ )
   {
      if ( !coeffs.size() )
      {
         error = "mt_mappedblur : unvalid kernel";
         return;
      }
      matrix[9] += abs(matrix[i] = Word(coeffs.front().getValue(0,0,0)));
      coeffs.pop_front();
   }

   if ( coeffs.size() )
      matrix[9] = Word(coeffs.front().getValue(0,0,0));

   if ( !matrix[9] )
      matrix[9] = 1;

   /* add the processors */
   if ( parameters["mode"].toString() == "replace" )
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mapped_all_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   else if ( parameters["mode"].toString() == "dump" )
      addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mapped_below_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));

}

String Filter::getName() { return "mt_mappedblur"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(String("1 1 1 1 1 1 1 1 1"), "kernel"));
   signature.push_back(Parameter(String("replace"), "mode"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::TwoFrames();

} } } }
