#include "filter.h"
#include "../../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Lut { namespace Dual {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nWidth, nHeight, yLut, prefetch);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nWidthUV, nHeightUV, uLut, prefetch);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nWidthUV, nHeightUV, vLut, prefetch);
}

Filter::Filter(const Parameters::Parameters &parameters) :
MaskTools::Filter(parameters)
{
   Parser::Parser parser = Parser::getDefaultParser().addSymbol(Symbol::X).addSymbol(Symbol::Y);

   prefetch = parameters["preload"].toBool() ? prefetches.best_processor(Constraint(flags, 1, 1, 1, 1)) : Functions::prefetch_nothing_c;

   print( LOG_ERROR, "prefetch : %i %i %i %i\n", prefetch, Functions::prefetch_nothing_c, Functions::prefetch_c, Functions::prefetch_isse );

   /* compute the luts */
   /* Y */
   if ( parameters["yExpr"].is_defined() ) parser.parse(parameters["yExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context yCtx(parser.getExpression());
   if ( !yCtx.check() )
   {
      error = "lutxy : invalid yExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      for ( int y = 0; y < 256; y++ )
         yLut[(x<<8)+y] = yCtx.compute_byte(x, y);

   /* U */
   if ( parameters["uExpr"].is_defined() ) parser.parse(parameters["uExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context uCtx(parser.getExpression());
   if ( !uCtx.check() )
   {
      error = "lutxy : invalid uExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      for ( int y = 0; y < 256; y++ )
         uLut[(x<<8)+y] = uCtx.compute_byte(x, y);

   /* V */
   if ( parameters["vExpr"].is_defined() ) parser.parse(parameters["vExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context vCtx(parser.getExpression());
   if ( !vCtx.check() )
   {
      error = "lutxy : invalid vExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      for ( int y = 0; y < 256; y++ )
         vLut[(x<<8)+y] = vCtx.compute_byte(x, y);

   /* add the processor */
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(&lut_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
}

String Filter::getName() { return "mt_lutxy"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(String("x"), "expr"));
   signature.push_back(Parameter(String("x"), "yExpr"));
   signature.push_back(Parameter(String("x"), "uExpr"));
   signature.push_back(Parameter(String("x"), "vExpr"));
   signature.push_back(Parameter(true, "preload"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::IPTwoFrames();

} } } }